<?php

/**
 * This is a super simple wrapper around the ascontrol.py python app that
 * talks to the command line interface (CLI) of pulseaduio. With this
 * I can control the audio switch settings.
 */

$python_loc = "/usr/bin/python ";
$control_app = $python_loc . "ascontrol-combine.py ";
$config_zone_command = $control_app . "config_zone ";
$default_config_command = $control_app . "set_default_config ";
$get_config_command = $control_app . "get_config ";
$get_volume_command = $control_app . "get_volume";
$set_volume_command = $control_app . "set_volume ";

/**
 * We expect as input an array of arrays, and we just dump the contents.
 * The general structure is that each inner array contains 3 elements,
 * the zone number, the player number, and the module index number, in
 * that order.
 */
function DumpConfig($config_array)
{
	foreach ($config_array as $zone)
	{
		foreach ($zone as $item)
		{
			echo $item . ", ";
			
		}
		echo "<br>";
	}
}

function GetVolumeInfo($zone_number)
{
	global $get_volume_command;
	
	$volume = RunCommand($get_volume_command);
	$volume = explode(",", $volume);
	
	if ($zone_number >= 0 && $zone_number < count($volume))
	{
		$vol = $volume[$zone_number];
		$vol = str_replace("[", "", $vol);
		$vol = str_replace("'", "", $vol);
		
		return (int)$vol;
	}
	else
		return 50;
}

function SetVolume($zone_number, $volume)
{
	global $set_volume_command;
	
	if ($volume >= 0 && $volume <= 100)
	{
		RunCommand($set_volume_command . ($zone_number) . " " . $volume);
	}
}

/**
 * Read the current config from pulseaudio, then look through the
 * zones to find the one requested, return that array of 3 numbers.
 */
function GetZoneInfo($zone_number)
{
	$config = ReadConfig();
	
	foreach ($config as $zone)
	{
		if ($zone[0] == $zone_number)
		{
			return $zone;
		}
	}
	
	return null;
}

/**
 * Here we take an array of 2 numbers, the zone and the player numbers
 * that we would like connected. We look up the zone to see if it is
 * already connected properly. If it is not we update the config.
 */
function SetZone($zone_info)
{
	global $config_zone_command;
	
	$zone = $zone_info[0];
	$player = $zone_info[1];
	$module_index = $zone_info[2];
	
	$current_zone_info = GetZoneInfo($zone);
	
	if ($current_zone_info != null)
	{
		if ($current_zone_info[1] != $player)
		{
			// its not a match, better update
			$command = $config_zone_command . $zone . " " . $player;
			RunCommand($command);
		}
	}
}

/**
 * The config that comes out of the CLI app is nothing more than the
 * python string representation of an array of arrays, so we have
 * to parse it into an array of arrays in php.
 */
function ParseConfigString($config)
{
	$pieces = explode("], [", $config);
	
	$config_array = array();
	foreach ($pieces as $piece)
	{
		$piece = str_replace("[[", "", $piece);
		$piece = str_replace("]]", "", $piece);
		$piece = rtrim($piece);
		$config_array[] = explode(", ", $piece);
	}

	return $config_array;
}

/**
 * This deals with starting up the control app and telling it what to do.
 * Its response is returned.
 */
function RunCommand($command)
{
	$descriptor = array(
		0 => array("pipe", "r"), // stdin
		1 => array("pipe", "w"), // stdout
		2 => array("pipe", "w"), // stderr
		);
		
	$pipes = array();
	$process = proc_open($command, $descriptor, $pipes);

	$output = stream_get_contents($pipes[1]);
	// get the errors
	//echo stream_get_contents($pipes[2]);
	
	fclose($pipes[0]);
	fclose($pipes[1]);
	fclose($pipes[2]);
	proc_close($process);
	
	return $output;
}

/**
 * Tell the control app to just put things back to their default zone/player
 * configuration.
 */
function SetDefaultConfig()
{
	global $default_config_command;
	RunCommand($default_config_command);
}

/**
 * Reads the current config for all zones and parses it so that the return is
 * a php friendly array of arrays.
 */
function ReadConfig()
{
	global $get_config_command;
	
	$config = RunCommand($get_config_command);
	
	return ParseConfigString($config);
}

/*
DumpConfig(ReadConfig());
echo "<br>";

SetZone(array("1", "4"));
//SetDefaultConfig();

echo "<br>";
DumpConfig(ReadConfig());
*/

?>